
import math
import matplotlib.pyplot as plt
import numpy as np

aspect = 1.0
area = 0.5

h = 0.015   # Plate thickness
nu = 0.3    # Poisson
E = 4.0E6
rho = 26    # density kg/m^3

D = E * h**3 /(12*(1-nu*nu))
K = math.pi/2 * math.sqrt(D/(rho*h))


def modal_freq(a2, b2, m, n):
    return K * (m*m/a2 + n*n/b2)

def calc_wavenumber_limits(a2, b2, fmax):
    '''
    Wavenumbers less than a fmax form an ellipse in k-space.
    Calculate the wavenumbers that enclose the ellipse
     '''
    mmax = int(math.floor(math.sqrt(a2*(fmax/K - 1/b2)))/2)
    nmax = int(math.floor(math.sqrt(b2*(fmax/K - 1/a2)))/2)
    #print(mmax,nmax)
    return (mmax, nmax)

def modes_less_than(a2, b2, fmax):
    '''
    Return a sorted list of modes less than fmax for a panel which 
    has side lengths a2, b2
   
    For simplicity, try all combinations of m,n within the enclosing 
    rectangle '''

    mmax, nmax = calc_wavenumber_limits(a2, b2, fmax)

    #print(nmax, mmax, K1 * nmax * nmax / a2 )
    freqs = []

    for m in range(1, mmax+1):
        for n in range(1, nmax+1):
            f = modal_freq(a2, b2, m, n)
            freqs.append(f)
        #print (freqs)
        freqs.sort()
        if freqs[-1] < fmax:
            print('Error: wavenumber limit too small')
        
    return [f for f in freqs if f < fmax]

def mode_aspect(aspects, n, m):
    '''Return a list of modal frequencies for one mode over a range of aspect 
    ratios'''

    modeFreqs = []
    for aspect in aspects:
        a2 = (area * aspect)
        b2 = (area/aspect)
        
        f = modal_freq(a2, b2, m, n)
        modeFreqs.append(f)
                         
    return modeFreqs
      


'''
aspects = numpy.linspace(1.0, 5.0, 50)
std_devs = []

allFreqs = []


for aspect in aspects:

    a2 = (area * aspect)
    b2 = (area/aspect)
    freqs = modes_less_than(a2, b2, 100)
    allFreqs.append(freqs)
    diffs = [freqs[i+1]-freqs[i] for i, _ in enumerate(freqs[0:-1]) ]
    mean = sum(diffs)/ len(diffs)
    sd = math.sqrt(sum([(x-mean)**2 for x in diffs]) / len(diffs))
    std_devs.append(sd)
    #print (len(freqs))
    #print (aspect, len(freqs), freqs[0], freqs[-1], max(diffs), mean, sd)                         

for i in range(0, 8):
    ax.plot(aspects, [freq[i] for freq in allFreqs])
                         
'''

def top_and_tail(list, max):
    '''
    Eliminate data too large for the plot
    '''

    i = 0
    N = len(list)

    while i < N and list[i] > max:
        i += 1
    j = N-1
    while j > i and list[j] > max:
        j -= 1

    if j <= i:
        return (0, 0)
    return (i, j) 

def on_plot_hover(event):
    # Iterating over each data member plotted
    for curve in ax.get_lines():
        # Searching which data member corresponds to current mouse position
        if curve.contains(event)[0]:
            s = curve.get_gid()
            mode = str(int(modeindex[s,1])) + "," + str(int(modeindex[s,2]))
            print("mode ", mode, "   ", end="\r")
            #print("over %s" % curve.get_gid())

fmax = 500
amax = 6.0
mLimit=0
nLimit=0

fig = plt.figure(figsize=(9, 9))
ax = fig.add_subplot(111)
amin = 1.0  # minimum aspect ratio
aspects = np.linspace(1.0, amax, 100)

# a and b are the height and width of the panel
# a is the larger dimension
# a2 is a^2, a2 is b^2

# Highest wavenumber on side b will be at minimum aspect ratio
a2 = (area * amin)
b2 = (area / amin)
(_, nmax) = calc_wavenumber_limits(a2, b2, fmax)

# Highest wavenumber on side a will be a max aspect ratio
a2 = (area * amax)
b2 = (area / amax)
(mmax, _) = calc_wavenumber_limits(a2, b2, fmax)

if mLimit > 0:
    mmax = mLimit
if nLimit > 0:
    nmax = nLimit

plt.ylabel('Frequency')
plt.xlabel('Aspect ratio')
plt.title('n=1 series')
aspectsA = np.array(aspects)
kcurve = 0
modeindex = np.zeros([(mmax+1)*(nmax+1),3])
for m in range(1, mmax+1):
    mm = 2*m - 1
    for n in range(1, nmax+1):
        kcurve = kcurve + 1
        nn = 2*n - 1
        modeindex[kcurve,:] = [kcurve, mm, nn]
        #print(mm, nn)
        #if m % 2 == 0 and n % 2 == 0:
        #    continue
        freqs = mode_aspect(aspects, nn, mm)
        i, j = top_and_tail(freqs, fmax)
        freqsA = np.array(freqs)
        if mm == 1 and nn == 1:
            freq11 = freqsA
            i1 = i
            j1 = j
        #ax.plot(aspects[i:j], freqs[i:j])
        lbl = str(mm)+","+str(nn)
        ax.semilogy(aspectsA[i:j], freqsA[i:j], gid=kcurve)
        #ax.semilogy(aspectsA[i:j], freqsA[i:j]/freq11[i:j], label=lbl)
#ax.legend()
fig.canvas.mpl_connect('motion_notify_event', on_plot_hover)
plt.show()



